@@ -3,6 +3,7 @@
  *
  * Released under the MIT License, as described in LICENSE.md in this repository
  */
+
 #if _WIN32
 #define _CRT_SECURE_NO_WARNINGS 1
 #endif
@@ -18,7 +19,13 @@
 #include <juce_core/system/juce_TargetPlatform.h>
 #include <juce_audio_processors/juce_audio_processors.h>
 #include <juce_gui_basics/juce_gui_basics.h>
+
+#if JUCE_VERSION >= 0x08000B
+#include <juce_audio_processors_headless/juce_audio_processors_headless.h>
+#include <juce_audio_processors_headless/format_types/juce_LegacyAudioParameter.h>
+#else
 #include <juce_audio_processors/format_types/juce_LegacyAudioParameter.cpp>
+#endif
 
 #if JUCE_VERSION >= 0x070006
 #include <juce_audio_plugin_client/detail/juce_IncludeSystemHeaders.h>
@@ -53,7 +60,10 @@ JUCE_END_IGNORE_WARNINGS_GCC_LIKE
 
 #if JUCE_LINUX
 #if JUCE_VERSION >= 0x070006
+#include <vector>
+#include <juce_events/native/juce_EventLoopInternal_linux.h>
 #include <juce_audio_plugin_client/detail/juce_LinuxMessageThread.h>
+#define HAS_LINUX_FD 1
 #elif JUCE_VERSION > 0x060008
 #include <juce_audio_plugin_client/utility/juce_LinuxMessageThread.h>
 #endif
@@ -240,8 +250,8 @@ class EditorContextMenu : public juce::HostProvidedContextMenu
                 item.itemID = ++menuIDCounter;
                 item.text = juce::CharPointer_UTF8(entry->label);
                 item.isEnabled = entry->is_enabled;
-                item.action = [&host = this->host, target = *this->menuTarget,
-                               id = entry->action_id] { host.contextMenuPerform(&target, id); };
+                item.action = [&hostRef = this->host, target = *this->menuTarget,
+                               id = entry->action_id] { hostRef.contextMenuPerform(&target, id); };
 
                 currentMenu.addItem(item);
             }
@@ -254,8 +264,8 @@ class EditorContextMenu : public juce::HostProvidedContextMenu
                 item.text = juce::CharPointer_UTF8(entry->label);
                 item.isEnabled = entry->is_enabled;
                 item.isTicked = entry->is_checked;
-                item.action = [&host = this->host, target = *this->menuTarget,
-                               id = entry->action_id] { host.contextMenuPerform(&target, id); };
+                item.action = [&hostRef = this->host, target = *this->menuTarget,
+                               id = entry->action_id] { hostRef.contextMenuPerform(&target, id); };
 
                 currentMenu.addItem(item);
             }
@@ -384,6 +394,9 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
                         public juce::AudioProcessorListener,
                         public juce::AudioPlayHead,
                         public juce::AudioProcessorParameter::Listener,
+#if HAS_LINUX_FD
+                        public juce::LinuxEventLoopInternal::Listener,
+#endif
                         public juce::ComponentListener
 {
   public:
@@ -492,6 +505,10 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
                 dynamic_cast<clap_juce_extensions::clap_juce_parameter_capabilities *>(juceParam)};
             clapIDByParamPtr[juceParam] = clapID;
         }
+
+#if HAS_LINUX_FD
+        juce::LinuxEventLoopInternal::registerLinuxEventLoopListener(*this);
+#endif
     }
 
     ~ClapJuceWrapper() override
@@ -501,6 +518,11 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
         {
             _host.timerSupportUnregister(idleTimer);
         }
+
+#if HAS_LINUX_FD
+        juce::LinuxEventLoopInternal::deregisterLinuxEventLoopListener(*this);
+        unregisterExtantFDs();
+#endif
 #endif
     }
 
@@ -517,10 +539,7 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
         return true;
     }
 
-    void reset() noexcept override
-    {
-        processor->reset();
-    }
+    void reset() noexcept override { processor->reset(); }
 
   public:
     bool implementsTimerSupport() const noexcept override { return true; }
@@ -546,6 +565,43 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
 #endif
     }
 
+#if HAS_LINUX_FD
+    std::vector<int> registeredFDs;
+    void fdCallbacksChanged() override
+    {
+        unregisterExtantFDs();
+        registeredFDs = juce::LinuxEventLoopInternal::getRegisteredFds();
+        if (_host.canUsePosixFdSupport())
+        {
+            for (auto &fd : registeredFDs)
+            {
+                _host.posixFdSupportRegister(fd, CLAP_POSIX_FD_READ | CLAP_POSIX_FD_WRITE | CLAP_POSIX_FD_ERROR);
+            }
+        }
+    }
+    void unregisterExtantFDs()
+    {
+        if (_host.canUsePosixFdSupport())
+        {
+            for (auto &fd : registeredFDs)
+            {
+                _host.posixFdSupportUnregister(fd);
+            }
+        }
+        registeredFDs.clear();
+    }
+
+    bool implementsPosixFdSupport() const noexcept override { return true; }
+    void onPosixFd(int fd, clap_posix_fd_flags_t /* flags */) noexcept override
+    {
+        juce::ScopedJuceInitialiser_GUI libraryInitialiser;
+        const juce::MessageManagerLock mmLock;
+
+        juce::LinuxEventLoopInternal::invokeEventLoopCallbackForFd(fd);
+    }
+
+#endif
+
     clap_id idleTimer{0};
 
     static uint32_t generateClapIDForJuceParam(juce::AudioProcessorParameter *param)
@@ -684,6 +740,7 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
     bool supressParameterChangeMessages{false};
     void audioProcessorParameterChanged(juce::AudioProcessor *, int index, float newValue) override
     {
+#if 0 // PARAM_LISTENERS_ON_MAIN_THREAD
         if (cacheHostCanUseThreadCheck)
         {
             // Parameter change messages should not be coming from the audio thread!
@@ -695,6 +752,7 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
                 return;
             }
         }
+#endif
 
         // This change message came from an event that we've already handled.
         // Let's return here to avoid creating a feedback loop!
@@ -793,10 +851,9 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
                                        CLAP_BEATTIME_FACTOR);
                 posinfo.setPpqPositionOfLastBarStart(1.0 * (double)transportInfo->bar_start /
                                                      CLAP_BEATTIME_FACTOR);
-                juce::AudioPlayHead::LoopPoints loopPoints {
-                    1.0 * (double) transportInfo->loop_start_beats / CLAP_BEATTIME_FACTOR,
-                    1.0 * (double) transportInfo->loop_end_beats / CLAP_BEATTIME_FACTOR
-                };
+                juce::AudioPlayHead::LoopPoints loopPoints{
+                    1.0 * (double)transportInfo->loop_start_beats / CLAP_BEATTIME_FACTOR,
+                    1.0 * (double)transportInfo->loop_end_beats / CLAP_BEATTIME_FACTOR};
                 posinfo.setLoopPoints(loopPoints);
             }
             if (flags & CLAP_TRANSPORT_HAS_SECONDS_TIMELINE)
@@ -1055,30 +1112,69 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
 
     bool implementsNoteName() const noexcept override
     {
+#if JUCE_VERSION < 0x080005
         if (processorAsClapExtensions)
             return processorAsClapExtensions->supportsNoteName();
-        return false;
+#endif
+
+        return true;
     }
 
     uint32_t noteNameCount() noexcept override
     {
-        if (processorAsClapExtensions)
+        if (processorAsClapExtensions && processorAsClapExtensions->supportsNoteName())
             return processorAsClapExtensions->noteNameCount();
+
+#if JUCE_VERSION >= 0x080005
+        noteNameInfoCached.clear();
+
+        // The JUCE docs say that plugins need to be able to handle
+        // note name requests for notes above 127, but CLAP only supports
+        // notes in range [0, 127].
+        for (int key = 0; key < 128; ++key)
+        {
+            // JUCE expects MIDI channels in range [1, 16].
+            // CLAP expects MIDI channels in range [0, 15].
+            for (int channel = 0; channel < 16; ++channel)
+            {
+                const auto optionalNoteName = processor->getNameForMidiNoteNumber(key, channel + 1);
+                if (optionalNoteName.has_value())
+                    noteNameInfoCached.push_back(
+                        {*optionalNoteName, (int16_t)key, (int16_t)channel});
+            }
+        }
+
+        return static_cast<uint32_t>(noteNameInfoCached.size());
+#else
         return 0;
+#endif
     }
 
     bool noteNameGet(uint32_t index, clap_note_name *noteName) noexcept override
     {
-        if (processorAsClapExtensions)
+        if (processorAsClapExtensions && processorAsClapExtensions->supportsNoteName())
             return processorAsClapExtensions->noteNameGet(index, noteName);
+
+#if JUCE_VERSION >= 0x080005
+        if (index >= noteNameInfoCached.size())
+            return false;
+
+        const auto &noteNameInfo = noteNameInfoCached[index];
+        noteNameInfo.name.copyToUTF8(noteName->name, CLAP_NAME_SIZE);
+        noteName->key = noteNameInfo.key;
+        noteName->channel = noteNameInfo.channel;
+        noteName->port = -1;
+        return true;
+#else
         return false;
+#endif
     }
 
     bool implementsTrackInfo() const noexcept override { return true; }
 
     void trackInfoChanged() noexcept override
     {
-        clap_track_info clapTrackInfo {};
+        clap_track_info clapTrackInfo{};
         if (_host.trackInfoGet(&clapTrackInfo))
         {
             juce::AudioProcessor::TrackProperties juceTrackInfo{};
@@ -1089,7 +1185,11 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
             }
             if (clapTrackInfo.flags & CLAP_TRACK_INFO_HAS_TRACK_COLOR)
             {
+#if JUCE_VERSION >= 0x08000B
+                juceTrackInfo.colourARGB = clapColourToJUCEColour(clapTrackInfo.color).getARGB();
+#else
                 juceTrackInfo.colour = clapColourToJUCEColour(clapTrackInfo.color);
+#endif
             }
 
             processor->updateTrackProperties(juceTrackInfo);
@@ -1389,10 +1489,10 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
         param.processorParam->setValue(newValue);
 
 #if 0 // PARAM_LISTENERS_ON_MAIN_THREAD
-        // we want to trigger the parameter listener callbacks on the main thread,
-        // but MessageManager::callAsync is not safe to call from the audio thread.
-        // This assumption though turns out to be wrong. We want to call the listener
-        // from the audio thread and leave it up to the pugin to be smart.
+      // we want to trigger the parameter listener callbacks on the main thread,
+      // but MessageManager::callAsync is not safe to call from the audio thread.
+      // This assumption though turns out to be wrong. We want to call the listener
+      // from the audio thread and leave it up to the pugin to be smart.
         audioThreadParamListenerQ.push(ParamListenerCall{param.processorParam, newValue});
         _host.requestCallback();
 #else
@@ -1401,13 +1501,12 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
             param.processorParam->sendValueChangedMessageToListeners(newValue);
         }
 #endif
-
     }
 
     void onMainThread() noexcept override
     {
 #if 0 // PARAM_LISTENERS_ON_MAIN_THREAD
-        // handle parameter change listener callbacks
+      // handle parameter change listener callbacks
         juce::ScopedValueSetter<bool> suppressCallbacks{supressParameterChangeMessages, true};
         ParamListenerCall listenerCall{};
         while (audioThreadParamListenerQ.pop(listenerCall))
@@ -1654,7 +1753,7 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
             if (hostCalledWithDouble)
             {
                 juce::AudioBuffer<double> buffer(doubleBusses.data(), (int)totalChans,
-                                                numSamplesToProcess);
+                                                 numSamplesToProcess);
 
                 if (processor->isSuspended())
                 {
@@ -1682,7 +1781,6 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
                 }
             }
 
-
             if (processorAsClapExtensions && processorAsClapExtensions->supportsOutboundEvents())
             {
                 processorAsClapExtensions->addOutboundEventsToQueue(process->out_events, midiBuffer,
@@ -2305,6 +2403,10 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
             return false;
         }
 
+        // On newer JUCE versions we can use chunkMemory.isEmpty(), but older JUCE didn't have that
+        if (chunkMemory.getSize() == 0)
+            return false;
+
         // JUCE has no way to report an unstream error; setStateInformation is void
         // So we just have to assume it works.
         processor->setStateInformation(chunkMemory.getData(), (int)chunkMemory.getSize());
@@ -2387,6 +2489,14 @@ class ClapJuceWrapper : public clap::helpers::Plugin<
 
     const clap_event_transport *transportInfo{nullptr};
     bool hasTransportInfo{false};
+
+    struct NoteNameInfo
+    {
+        juce::String name{};
+        int16_t key = -1;
+        int16_t channel = -1;
+    };
+    std::vector<NoteNameInfo> noteNameInfoCached{};
 };
 
 JUCE_END_IGNORE_WARNINGS_GCC_LIKE
